%%% THIS FUNCTION IS USED TO READ DATA AND PARAMETERS FROM A STRUCTURED
%%% POSITIONS. IT IS USED ON cluster.norijacoby.com for the clustering.
%%% to run locally use other "MAIN" functions 
clear all;%"To begin at the beginning, It is Spring, moonless night in the small town..." 
if strcmp(getenv('OS'),'Windows_NT')
    VERBOSE=10.5; %print and draw graph only in windows
else
    VERBOSE=.5;   %only run results
end

fname_data='input/joint_histogram.csv'; % fixed name check if exist
fname_pre_determined_clusters='input/pre_determined_clusters.csv';

fname_param='input/params.txt';
fname_error='output/error.txt';
fname_summary='output/summary.txt';
fname_general='output/general.txt';

directory_for_output='output'; %if empty there will be no outpu

try
    [param,msg]=IBLIB_read_params_from_file(fname_param);
    if ~isempty(msg)
        IBLIB_write_error(fname_error,msg);
        return;
    else
        display(sprintf('ok reading parameters'));
    end
    
    ITER_infocurve=param.ITER_infocurve;
    ITER_clusters=param.ITER_clusters;
    %ITER_infocurve=20;  % How many iterations (fast).
    %ITER_clusters=40;   % How many iterations on optimal clusters (fast).
    %ITER_infocurve=100; % How many iterations (slow).
    %ITER_clusters=200;  % How many iterations on optimal clusters (slow).
    
    
    KS=param.KS;
    KS_betaS=param.KS_betaS;
    %infocurve_betaS=max(param.beta_range):(-abs(param.beta_jump)):min(param.beta_range);
    infocurve_betaS=2.^((log2(max(param.beta_range)):(-abs(param.beta_jump)):log2(min(param.beta_range))));
    
    
    %KS=[2,3,4,5,6,7];% list of deterministic clusters to compute
    %KS_betaS=[]; % the clusters betas (should be a large number) set default values if empty
    %infocurve_betaS=[]; % set to default values if empty
    
    name_of_dataset='Dataset'; % not really used with VERBOSE small
    
    CNAME=name_of_dataset;
    
    
    FFS=fopen(fname_summary,'w+');
    if FFS==-1
        msg=sprintf('can''t write to summary file %s',fname_summary);
        IBLIB_write_error(fname_error,msg);
    end
    fprintf(FFS,'Summary file\n------------------\nDate:');
    fprintf(FFS,date);
    fprintf(FFS,'\nParameters:\n')
    fprintf(FFS,sprintf('ITER_infocurve:\t%d\n',ITER_infocurve));
    fprintf(FFS,sprintf('ITER_clusters:\t%d\n',ITER_clusters));
    fprintf(FFS,strcat('KS:  ',sprintf(' %d ',KS)));
    fprintf(FFS,'\n');
    fprintf(FFS,strcat('KS_beta:  ',sprintf(' %g ',KS_betaS)));
    fprintf(FFS,'\n');
    fprintf(FFS,sprintf('beta-range:\t from %g to %g\n',max(infocurve_betaS),min(infocurve_betaS)));
    fprintf(FFS,sprintf('beta-jump (in log2-scale):\t %g\n',param.beta_jump));
    fprintf(FFS,'------------------\n');
    TSTART = tic;
    
    %%% end of remove zone if this is not a script
    %%% ****************************************************************************************
    
    if isempty(KS_betaS)
        KS_betaS=nan(size(KS));
        for I=1:length(KS)
            if KS(I)==2
                KS_betaS(I)=120;
            else
                KS_betaS(I)=150;
            end
        end
    end
    
    % info curve betas...
    if isempty(infocurve_betaS)
        mybetaS=2.^((log2(80):(-.1):log2(1.5)));
    else
        mybetaS=infocurve_betaS;
    end
    if ~isempty(directory_for_output)
        session_header_forfiles=sprintf('%s/',directory_for_output); % if empty no output
    else
        session_header_forfiles=[];
    end
    
    
    [msg,NAMESX,NAMESY,NUM,NAMESZ,CATZ_NAMEs,CATZs,NX,NY,NZ,pyx]=IBLIB_read_data_files(fname_data,fname_pre_determined_clusters);
    if NZ==0
        display(sprintf('No pre-determined clusters...'));
    end
    
    if ~isempty(msg)
        IBLIB_write_error(fname_error,msg);
        return;
    else
        display(sprintf('ok reading and processing data!'));
    end
    
    
    if VERBOSE>=14
        figure(1);
        imagesc(1:NX,1:NY,log2(pyx+1));
        set(gca,'Xtick',1:NX);
        set(gca,'XtickLabel',NAMESX);
        set(gca,'Ytick',1:NY);
        set(gca,'YtickLabel',NAMESY);
        
    end
    
    
    % pyx=pyx/sum(sum(pyx));
    % x=sum(pyx)';
    % y=sum(pyx,2);
    % NX=length(x);
    % NY=length(y);
    % PYX=pyx.*repmat(1./(x'+eps),[size(pyx,1),1]);
    % PCX0=ones(NX,NX);
    % PTY0=eye(length(y));
    
    %normalize histograms
    display(sprintf('normalize data...'));
    pyx=pyx/sum(sum(pyx));
    assert(abs(sum(sum(pyx))-1)<1e-5);
    x=sum(pyx)';
    y=sum(pyx,2);
    PYX=pyx.*repmat(1./(x'+eps),[size(pyx,1),1]);
    PCX0=ones(NX,NX);
    PCX0=eye(length(x));
    PTY0=eye(length(y));
    
    display(sprintf('computing predetermined clusters...'));
    [tPCXs,tPYCXs,tPYCs,tPTCs,tICXs,tIYCs,tIYX,FUNCZs] = IBLIB_run_predeterminedclusters(PYX,x,y,NAMESX,CATZs,CATZ_NAMEs,session_header_forfiles);
    if NZ>0
        INFOPOINTS=[tICXs,tIYCs];
        fprintf(FFS,'\nPredetermined clusters (short format):\n');
        for I=1:NZ,
            [msg,~]=IBLIB_print_clusters(tPCXs{I},x,NAMESX,0.1,false,FUNCZs{I});
            fprintf(FFS,sprintf('predetermined cluster: %s  complexity= %g accuracy= %g \t %s \n',CATZ_NAMEs{I},INFOPOINTS(I,1),INFOPOINTS(I,2),msg));
        end
        fprintf(FFS,'\nPredetermined clusters (long format):\n');
        for I=1:NZ,
            [~,~,msgBB]=IBLIB_print_clusters(tPCXs{I},x,NAMESX,0.1,1,FUNCZs{I});
            fprintf(FFS,sprintf('predetermined cluster: %s  complexity= %g accuracy= %g \t %s \n',CATZ_NAMEs{I},INFOPOINTS(I,1),INFOPOINTS(I,2),msgBB));
        end
        fprintf(FFS,'\n');
        
    else
        fprintf(FFS,'\nNOTE: No Predetermined clusters (no input or bad input file)\n');
    end
    if KS==0
        fprintf(FFS,'\nNOTE: No computation of deterministic clusters (input parameter was empty)\n');
    end
    %%%%% writing general info file...
    FID=fopen(fname_general,'w+');
    if FID==-1
        msg=sprintf('can''t write to general parameter file %s',fname_summary);
        IBLIB_write_error(fname_error,msg);
    end
    fprintf(FID,sprintf('pre-clusters-number,%d\n',NZ));
    fprintf(FID,sprintf('ks-number,%d\n',length(KS)));
    fprintf(FID,sprintf('beta-number,%d\n',length(mybetaS)));
    fprintf(FID,sprintf('ks,%s\n', sprintf('%d ',KS)));
    fprintf(FID,sprintf('betas,%s\n', sprintf('%g ',mybetaS)));
    fclose(FID);
    
    %%%%
    
    pertub_probS=[];
    temperatureS=[];
    what_to_do=[];
    
    display(sprintf('computing info curve...'));
    ITER=ITER_infocurve;
    [PCXs,PTYs,PTCs,PYCs,bestLs,listLs,ICXs,ITYs,ITCs,IYCs,IYX]=IBLIB_run_batch_betas(x,y,PCX0,PTY0,PYX,ITER,pertub_probS,temperatureS,what_to_do,VERBOSE,mybetaS,NAMESX,session_header_forfiles);
    display(sprintf('doing plots (if needed)...'));
    
    betaS=KS_betaS;
    ITER=ITER_clusters;
    if ~isempty(KS)
        [kPCXs,kPTYs,kPTCs,kPYCs,kbestLs,klistLs,kICXs,kITYs,kITCs,kIYCs,kIYX]=IBLIB_run_batch_cluster_sizes(x,y,PYX,ITER,pertub_probS,temperatureS,what_to_do,VERBOSE,KS,betaS,NAMESX,session_header_forfiles);
    else
        kPCXs={};kPTYs={};kPTCs={};kPYCs={};kbestLs=[];klistLs={};kICXs=[];kITYs=[];kITCs=[];kIYCs=[];kIYX=[];
    end
    
    display(sprintf('computing optimal clusters...'));
    if VERBOSE>=0.5
        for I=1:length(KS)
            IBLIB_print_clusters(kPCXs{I},x,NAMESX,0.1,1,[]);
        end
    end
    
    if VERBOSE>=10
        display(sprintf('doing plots (if needed)...'));
        IBLIB_doplots_script;
    end
    
    
    display(sprintf('saving summary information...'));
    if ~isempty(session_header_forfiles)
        display(sprintf('saving results to files... in location: %s',session_header_forfiles));
        %INFOPOINTS=[ICXs,IYCs,IYX*ones(size(ICXs)),mybetaS'];
        %dlmwrite(sprintf('%sfinaldata.info.curve.csv',session_header_forfiles),INFOPOINTS,'delimiter',',','precision',5);
        %INFOPOINTS=[kICXs,kIYCs,KS'];
        %dlmwrite(sprintf('%sfinaldata.info.optimalclusters.csv',session_header_forfiles),INFOPOINTS,'delimiter',',','precision',5);
        
        % output active tokens
        fname=sprintf('%sactivetokens.csv',session_header_forfiles);
        FID=fopen(fname,'w+');
        for I=1:length(NAMESX)
            fprintf(FID,'%s\n',NAMESX{I});
        end
        fclose(FID);
        
        fprintf(FFS,'\nDeterministic clusters (short format):\n');
        for I=1:length(KS)
            [msg,~]=IBLIB_print_clusters(kPCXs{I},x,NAMESX,0.1,false,[]);
            %          fname=sprintf('%sfinaldata.optimalclusters_strings.%d.csv',session_header_forfiles,KS(I));
            %          FID=fopen(fname,'w+');
            %          fprintf(FID,'%s',msg);
            %          fclose(FID);
            fprintf(FFS,sprintf('deterministic clusters for K=%d complexity= %g accuracy= %g \t %s\n',KS(I),kICXs(I),kIYCs(I),msg));
        end
        
        fprintf(FFS,'\nDeterministic clusters: (long format)\n');
        
        for I=1:length(KS)
            [~,~,msgBB]=IBLIB_print_clusters(kPCXs{I},x,NAMESX,0.1,1,[]);
            fprintf(FFS,sprintf('deterministic clusters for K=%d complexity= %g accuracy= %g \t %s\n',KS(I),kICXs(I),kIYCs(I),msgBB));
        end
        fprintf(FFS,'\n');
        
        
        fprintf(FFS,'------------------\n');
        fprintf(FFS,sprintf('\nActive tokens [N=%d]:\n',length(NAMESX)));
        for I=1:length(NAMESX)
            fprintf(FFS,'%s ',NAMESX{I});
        end
        fprintf(FFS,'\n');
        
        fprintf(FFS,'\n------------\nInformation curve [accuracy,complexity,maximal accuracy I(X;Y)]:\n');
        INFOPOINTS=[ICXs,IYCs,IYX*ones(size(ICXs))];
        for I=1:size(INFOPOINTS,1)
            fprintf(FFS,'%5.5g\t%5.5g\t%5.5g\n',INFOPOINTS(I,1),INFOPOINTS(I,2),INFOPOINTS(I,3));
        end
        fprintf(FFS,'----------------\n');
        
        %save(sprintf('%sdebugmatlabproject.mat',session_header_forfiles));
        
        elapsedTime = toc(TSTART);
        fname=sprintf('%sDONE.csv',session_header_forfiles);
        FID=fopen(fname,'w+');
        fprintf(FID,'\ntotal computation time %g sec\n',elapsedTime);
        fprintf(FID,'\nDONE Successfully!!');
        fclose(FID);
    end
    
    elapsedTime = toc(TSTART);
    fprintf(FFS,'\ntotal computation time: %g sec\n',elapsedTime);
    fprintf(FFS,'\nDONE Successfully!!');
    fclose(FFS);
    display(sprintf('DONE EVERYTHING!!!!'));
    
catch ME
    msg=sprintf('ERROR IN MATLAB SCRIPT (check the format of input files and input parameters), MATLAB ERROR (for debugging):\n%s' ,getReport(ME));
    IBLIB_write_error(fname_error,msg);
    %save(sprintf('%sdebugmatlabproject_error.mat',session_header_forfiles));
    if strcmp(getenv('OS'),'Windows_NT')
        fprintf('in windows not exiting...\n');
    else
        fprintf('not in windows now exiting...\n');
        exit(0);
    end
    
    %     return;
end

%% this should be done to terminate operation, not that when doing it in windows this quits MATLAB in a nasty way...
if strcmp(getenv('OS'),'Windows_NT')
    fprintf('in windows not exiting...\n');
else
    fprintf('not in windows now exiting...\n');
    exit(1);
end
