% This function compute the Tishby et al. solution to the IB problem
% Using a batch of beta values. The main use for this function is to sacan
% multiple beta values and obtain the information curve.
% x, y are the marginals; PCX0 initial cluster assignment also used to
% determine number of clusters (size(PCX0,1))
% PTY0 - not used here.
% PYX- P(Y|X)
% ITER number of total iteration per condition, total number of iteration
% is actually ITER*|Pertub_prob|*|what_to_do
% what_to_do- list of numbers like 1,2,3 where 1= start with original
% solution,2- permute original solution 3- randomize from scrach
% VERBOSE verbosity level, higher numbers more prints (typical value 13,10.5 and 0.5)
% mybetaS list of beta to work
% NAMESX- string array with names of surface tokens (used for printing only).
function [PCXs,PTYs,PTCs,PYCs,bestLs,listLs,ICXs,ITYs,ITCs,IYCs,IYX]=IBLIB_run_batch_betas(x,y,PCX0,PTY0,PYX,ITER,pertub_probS,temperatureS,what_to_do,VERBOSE,mybetaS,NAMESX,session_header_forfiles)
DRAW_COND=false;

if isempty(pertub_probS)
    pertub_probS=[0.1,0.9];
end
if isempty(temperatureS)
    temperatureS=[1e-5,100,1];
end
if isempty(what_to_do)
    what_to_do=[1,2,3];
end
if isempty(ITER)
    ITER=100;
end
if isempty(VERBOSE)
    VERBOSE=11;
end

myVERBOSE=mod(VERBOSE,10);

NB=length(mybetaS);
PCXs=cell(NB,1);
PTYs=cell(NB,1);
PTCs=cell(NB,1);
PYCs=cell(NB,1);
ICXs=zeros(NB,1);
ITYs=zeros(NB,1);
ITCs=zeros(NB,1);
IYCs=zeros(NB,1);

bestLs=zeros(NB,1);
listLs=cell(NB,1);
PCX=PCX0;
PTY=PTY0;

for k=1:NB,
    mybeta=mybetaS(k);
    if VERBOSE>=0.5
        display(sprintf('...computing information curve for beta=%g (%d of %d)...',mybeta,k,NB))
    end
    [PCX,PTY,PTC,PYC,bL,Ls]=IBLIB_do_annealing(x,y,PCX,PTY,PYX,mybeta,ITER,pertub_probS,temperatureS,what_to_do,myVERBOSE,NAMESX);
    PCXs{k}=PCX;
    PTYs{k}=PTY;
    if (isempty(PTC))
        PTC=0*PYX;
    end
    PTCs{k}=PTC;
    PYCs{k}=PYC;
    bestLs(k)=bL;
    listLs{k}=Ls;
    
    [ICX,ITY,ITC,IYC,IYX]=IBLIB_calc_info(x,y,PCXs{k},PTYs{k},PYX);
    ICXs(k)=ICX;
    ITYs(k)=ITY;
    ITCs(k)=ITC;
    IYCs(k)=IYC;
    c=PCX*x;
    t=PTY*y;
    PYCX=(PYC*PCX).*repmat(x',[size(PYX,1) 1]);
    
    if ~isempty(session_header_forfiles) % save temporary files...
        if k==1
            dlmwrite(sprintf('%sbeta.PYX.csv',session_header_forfiles),PYX,'delimiter',',','precision',5);
        end
        
        msg2=sprintf('%d',k);msgk='00000';msgk((end-length(msg2)+1):end)=msg2(:);
        dlmwrite(sprintf('%sbeta.%s.PCX.csv',session_header_forfiles,msgk),PCX,'delimiter',',','precision',3);
        dlmwrite(sprintf('%sbeta.%s.PTC.csv',session_header_forfiles,msgk),PTC,'delimiter',',','precision',3);
        dlmwrite(sprintf('%sbeta.%s.PCYX.csv',session_header_forfiles,msgk),PYCX,'delimiter',',','precision',3);
        if length(Ls)>2000
            Ls_toprint=[Ls(1:1000),ones(1,30)*mean(Ls),Ls((end-1000):end)];
        else
            Ls_toprint=Ls;
        end
        dlmwrite(sprintf('%sbeta.%s.scores.csv',session_header_forfiles,msgk),Ls_toprint,'delimiter',',','precision',3);
        INFOPOINTS=[ICXs(1:k),IYCs(1:k),IYX*ones(size(ICXs(1:k)))];
        dlmwrite(sprintf('%sbeta.%s.tempinfopoints.csv',session_header_forfiles,msgk),INFOPOINTS,'delimiter',',','precision',5);
        
        FIT=fopen(sprintf('%sbeta.%s.string.csv',session_header_forfiles,msgk),'w+');
        fprintf(FIT,'beta= %4.4g (%2d of %2d)\t complexity= %4.4g\t accuracy= %4.4g\t(total accuracy= %4.4g bits)\n',mybetaS(k),k,NB,ICX,IYC,IYX);
        fclose(FIT);
    end
    
    if VERBOSE>=10
        c=PCX*x;
        t=PTY*y;
        [~,idxC]=sort(c);
        [~,idxT]=sort(t);
        
        figure(10);clf;
        
        subplot(3,2,1);
        %imagesc(PCX(idxC,:),[0 1]);axis xy;
        imagesc(PCX(:,:));axis xy;
        title(sprintf('PCX(clusters assignment)'));
        
        subplot(3,2,2);
        if ~isempty(PTC)
            %imagesc(PTC(idxC,idxT),[0 1]);axis xy;
            imagesc(PTC(:,:));axis xy;
            title('PTC(cluster to next cluster)');
        end
        
        subplot(3,2,3);
        if DRAW_COND
            imagesc(PYC*PCX,[0 1]);axis xy;
            title('PYCX (prediction)');
        else
            imagesc((PYC*PCX).*repmat(x',[size(PYX,1) 1]));axis xy;
            title('P_C(Y,X) (predicted distribution)');
        end
        
        subplot(3,2,4);
        if DRAW_COND
            imagesc(PYX,[0 1]);axis xy;
            title('PYX');
        else
            imagesc(PYX.*repmat(x',[size(PYX,1) 1]));axis xy;
            title('P(Y,X) (real distribution)');
        end
        
        subplot(3,2,5);
        plot(1:length(Ls),Ls,'b-','LineWidth',2);hold on;
        plot(1:length(Ls),bL,'r-','LineWidth',2);
        title(sprintf('Lagrangian  values for beta=%g',mybeta));
        xlabel('iteration');
        ylabel('Lagrangian values');
        
        subplot(3,2,6);
        plot(ICXs(1:k),IYCs(1:k),'bo-','LineWidth',2);hold on;
        plot([0,max(ICXs)],[IYX IYX], 'g-','LineWidth',3);
        axis([0,max(ICXs),0, IYX]);
        xlabel('I(X;C)');
        ylabel('I(Y;C)');
        title('Information plane');
        
        drawnow;
    end
end

INFOPOINTS=[ICXs,IYCs,IYX*ones(size(ICXs)),mybetaS'];
if ~isempty(session_header_forfiles)
    dlmwrite(sprintf('%sbeta.infocurve.DONE.csv',session_header_forfiles),INFOPOINTS,'delimiter',',','precision',5);
end

